/*
 * CMLReader for InChI
 * Developed in collaboration with NIST and IUPAC
 * Version 1.0
 * Software version 1.0
 * 2004
 * Developed by Peter Murray-Rust and Simon (Billy) Tyrrell
 * Unilever Centre for Molecular Sciences Informatics
 * Department of Chemistry, University of Cambridge
 * Cambridge, CB2 1EW, UK
 *
 * CMLReader is free software 
 * you can redistribute this software and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation:
 * http://www.opensource.org/licenses/lgpl-license.php
 */


#ifndef CHARSX_H
#define CHARSX_H

// characters
static const char* const CH_AMP			= "&";
static const char* const CH_APOS 		= "'";
static const char* const CH_COLON		= ":";
static const char* const CH_EMPTY		= "";
static const char* const CH_EQUALS 	= "=";
static const char* const CH_HASH 		= "#";
static const char* const CH_LANGLE 	= "<";
static const char* const CH_NEWLINE	= "\n";
static const char* const CH_QUERY		= "?";
static const char* const CH_QUOTE 	= "\"";
static const char* const CH_RANGLE	= ">";
static const char* const CH_SLASH		= "/";
static const char* const CH_SPACE		= " ";

#endif // CHARSX_H

