; With gcc under Win32 this produces same exported names, ordinal numbers, and
; calling conventions as libinchi.dll produced by the MS VC++ 6.0 project
LIBRARY
    libinchi.dll
EXPORTS
; -- Exported name -------------------  Internal name ---------------- ordinal
     CheckINCHI                       = CheckINCHI                         @1
     CheckINCHIKey                    = CheckINCHIKey                      @2
     FreeINCHI                        = FreeINCHI                   	   @3
     FreeStdINCHI                     = FreeStdINCHI                       @4
     FreeStructFromINCHI              = FreeStructFromINCHI                @5
     FreeStructFromStdINCHI           = FreeStructFromStdINCHI      	   @6
     Free_inchi_Input                 = Free_inchi_Input                   @7
     Free_std_inchi_Input             = Free_std_inchi_Input               @8
     GetINCHI                         = GetINCHI                           @9
     GetINCHIKeyFromINCHI             = GetINCHIKeyFromINCHI               @10
     GetINCHIfromINCHI                = GetINCHIfromINCHI                  @11
     GetStdINCHI                      = GetStdINCHI                        @12
     GetStdINCHIKeyFromStdINCHI       = GetStdINCHIKeyFromStdINCHI         @13
     GetStringLength                  = GetStringLength                    @14
     GetStructFromINCHI               = GetStructFromINCHI                 @15
     GetStructFromStdINCHI            = GetStructFromStdINCHI              @16
     Get_inchi_Input_FromAuxInfo      = Get_inchi_Input_FromAuxInfo        @17
     Get_std_inchi_Input_FromAuxInfo  = Get_std_inchi_Input_FromAuxInfo    @18
     INCHIGEN_Create                  = Get_std_inchi_Input_FromAuxInfo    @19
     INCHIGEN_Destroy                 = INCHIGEN_Destroy                   @20
     INCHIGEN_DoCanonicalization      = INCHIGEN_DoNormalization           @21
     INCHIGEN_DoNormalization         = INCHIGEN_DoNormalization           @22
     INCHIGEN_DoSerialization         = INCHIGEN_DoSerialization           @23
     INCHIGEN_Reset                   = INCHIGEN_Reset                     @24
     INCHIGEN_Setup                   = INCHIGEN_Reset                     @25
     STDINCHIGEN_Create               = STDINCHIGEN_Create                 @26
     STDINCHIGEN_Destroy              = STDINCHIGEN_Create                 @27
     STDINCHIGEN_DoCanonicalization   = STDINCHIGEN_DoCanonicalization     @28
     STDINCHIGEN_DoNormalization      = STDINCHIGEN_DoNormalization        @29
     STDINCHIGEN_DoSerialization      = STDINCHIGEN_DoSerialization        @30
     STDINCHIGEN_Reset                = STDINCHIGEN_Reset                  @31
     STDINCHIGEN_Setup                = STDINCHIGEN_Setup                  @32

; ----------------- Pascal/stdcall calling convenions ------------------------
     _CheckINCHI@8                     = pasc_CheckINCHI@8                 @33
     _CheckINCHIKey@4                  = pasc_CheckINCHIKey@4              @34
     _FreeStdINCHI@4                   = pasc_FreeStdINCHI@4               @35
     _FreeStdINCHI@4                   = pasc_FreeStdINCHI@4               @36
     _FreeStructFromINCHI@4            = pasc_FreeStructFromINCHI@4        @37
     _FreeStructFromStdINCHI@4         = pasc_FreeStructFromStdINCHI@4     @38
     _Free_inchi_Input@4               = pasc_FreeStructFromStdINCHI@4     @39
     _Free_std_inchi_Input@4           = pasc_Free_std_inchi_Input@4       @40
     _GetINCHI@8                       = pasc_GetINCHI@8                   @41
     _GetINCHIKeyFromINCHI@24          = pasc_GetINCHIKeyFromINCHI@24      @42
     _GetINCHIfromINCHI@8              = pasc_GetINCHIfromINCHI@8          @43 
     _GetStdINCHI@8                    = pasc_GetStdINCHI@8                @44
     _GetStdINCHIKeyFromStdINCHI@8     = pasc_GetStdINCHIKeyFromStdINCHI@8  @45
     _GetStringLength@4                = pasc_GetStringLength@4            @46
     _GetStructFromINCHI@8             = pasc_GetStructFromINCHI@8         @47
     _GetStructFromStdINCHI@8          = pasc_GetStructFromStdINCHI@8      @48
     _Get_inchi_Input_FromAuxInfo@16   = pasc_Get_inchi_Input_FromAuxInfo@16 @49
     _Get_std_inchi_Input_FromAuxInfo@12 = pasc_Get_std_inchi_Input_FromAuxInfo@12 @50
     _INCHIGEN_Create@0                = pasc_INCHIGEN_Create@0            @51
     _INCHIGEN_Destroy@4               = pasc_INCHIGEN_Destroy@4           @52
     _INCHIGEN_DoCanonicalization@8    = pasc_INCHIGEN_DoCanonicalization@8 @53
     _INCHIGEN_DoNormalization@8       = pasc_INCHIGEN_DoNormalization@8   @54
     _INCHIGEN_DoSerialization@12      = pasc_INCHIGEN_DoSerialization@12  @55
     _INCHIGEN_Reset@12                = pasc_INCHIGEN_DoSerialization@12  @56
     _INCHIGEN_Setup@12                = pasc_INCHIGEN_Setup@12     	   @57
     _STDINCHIGEN_Create@0             = pasc_STDINCHIGEN_Create@0         @58
     _STDINCHIGEN_Destroy@4            = pasc_STDINCHIGEN_Destroy@4        @59
     _STDINCHIGEN_DoCanonicalization@8 = pasc_STDINCHIGEN_DoCanonicalization@8 @60
     _STDINCHIGEN_DoNormalization@8    = pasc_STDINCHIGEN_DoNormalization@8    @61
     _STDINCHIGEN_DoSerialization@12   = pasc_STDINCHIGEN_DoSerialization@12   @62
     _STDINCHIGEN_Reset@12             = pasc_STDINCHIGEN_Reset@12     	   @63
     _STDINCHIGEN_Setup@12             = pasc_STDINCHIGEN_Setup@12         @64
